%
% Section 4.2(a) Preprocessing 
%
% 2011/02/25 Modified
% 2010/07/28 Okito Yamashita
%
% Copyright (c) 2011, ATR Okito Yamashita, All rights reserved.

fprintf('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');
fprintf(' (a) Preprocessing of raw continuous EEG data. \n');
fprintf('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');


clear all
close all

doplot = 1; 

%---------------------------------------
% Set parameters for preprocessing EEG
%---------------------------------------

% Parameters for filenames
saveFileName = ['DATA' filesep 'trialsData.mat'];
loaddir = 'DATA';
loadFileName_prefix = 'rawdata';  % rawdata{01-07}.mat

% Parameter for digital filtering
lowpassFilterType = 'butter';
lowpassFilterOrder = 3;
lowpassFreq = 50;   % Hz
highpassFilterType = 'butter';
highpassFilterOrder = 3;
highpassFreq = 0.5; % Hz

% Parameters for trial segmentation
task1Status= 49;  % right
task2Status= 50;  % left
from_sec=-2;  %sec
to_sec=3.5 ;  %sec

% Constant variables (for allocating memory before)
SAMPFREQ = 256;
NCH      = 68;  % including 2 EMG and 2 EOG channels
NTR1     = 15;  % number of right hand trials per a run
NTR2     = 15;  % number of left hand trials per a run
NFILE    = 7;
NTRALL   = (NTR1+NTR2)*NFILE;  % number of total trials

from = from_sec*SAMPFREQ;
to   = to_sec*SAMPFREQ;
Nt = to - from;

% Allocate memory for saved variables 
data3 = zeros(NCH, Nt, NTRALL);
status3 = zeros(1,Nt,NTRALL);
trialLabel = zeros(NTRALL,1);
runLabel = zeros(NTRALL,1);
tr = 1;
Log = struct();

for nFile=1:NFILE;
    
    loadFileName = [loaddir filesep 'rawdata' num2str(nFile, '%02d') '.mat']
    
    %Load 
    load(loadFileName, 'data', 'time_sec', 'import');
    eeg    = data(import.ch_eeg, :);  % extract eeg channels only
    eeg(65,:) = data(65,:)-data(66,:);
    eeg(66,:) = data(67,:)-data(68,:);
    eeg(67,:) = data(69,:)-data(70,:);
    eeg(68,:) = data(71,:)-data(72,:);
    
    status = get_status_mask(data(import.ch_status, :)); % 
    

    if doplot
    ploteeg(time_sec, eeg(1:64,:), 'Raw Data');
    pause(0.5)
    end
    
    % Zero reset using the first 100 time points
    eeg = pre_zeroreset(eeg,1:100);
    
    if doplot
    ploteeg(time_sec, eeg(1:64,:), 'Zero Reset');
    pause(0.5)
    end
    
    % Highpass filter
    eeg = pre_highpass(eeg, SAMPFREQ, highpassFreq, highpassFilterType, highpassFilterOrder);
    
    if doplot
    ploteeg(time_sec, eeg(1:64,:), 'Highpass');
    pause(0.5)
    end
    
    % Lowpass filter
    eeg = pre_lowpass(eeg, SAMPFREQ, lowpassFreq, lowpassFilterType, lowpassFilterOrder);
    
    if doplot
    ploteeg(time_sec, eeg(1:64,:), 'Lowpass');
    pause(0.5)
    end
    
    % Common average reference
    eeg(1:64,:) = pre_car(eeg(1:64,:)); 

    if doplot
    ploteeg(time_sec, eeg, 'CAR');
    pause(0.5)
    end
     
    % Segment continuous data into trial data
    [eeg_seg] = pre_segmentdata(eeg, status, task1Status, from, to);
    data3(:,:,tr:tr+NTR1-1) = eeg_seg;
    trialLabel(tr:tr+NTR1-1) = 1;
    runLabel(tr:tr+NTR1-1) = nFile;
    
    [status_seg] = pre_segmentdata(status, status, task1Status, from, to);
    status3(:,:,tr:tr+NTR1-1) = status_seg;

    
    tr = tr + NTR1;

    [eeg_seg] = pre_segmentdata(eeg, status, task2Status, from, to);
    data3(:,:,tr:tr+NTR2-1) = eeg_seg;
    trialLabel(tr:tr+NTR2-1) = 2;
    runLabel(tr:tr+NTR2-1) = nFile;
    
    [status_seg] = pre_segmentdata(status, status, task1Status, from, to);
    status3(:,:,tr:tr+NTR1-1) = status_seg;
    
    tr = tr + NTR2;
    
    % Log information
    Log.import_parm(nFile) = import;
    loadFileNames{nFile} = loadFileName;
    
end

time_ms = [from+1:to]/SAMPFREQ*1000;
time_ms = time_ms + 1000; % adjust time_ms = 0 as imagery onset

pre_parm.comment{1} = '1 : right hand imagery, 2 : left hand imagery'
pre_parm.comment{2} = '1:64 EEG, 65 L-EMG, 66 R-EMG, 67 H-EOG, 68 V-EOG'
pre_parm.lowpassFilterType  = lowpassFilterType;
pre_parm.lowpassFilterOrder = lowpassFilterOrder;
pre_parm.lowpassFreq        = lowpassFreq;   % Hz
pre_parm.highpassFilterType  = highpassFilterType;
pre_parm.highpassFilterOrder = highpassFilterOrder;
pre_parm.highpassFreq        = highpassFreq; % Hz
pre_parm.loadFileNames       = loadFileNames;
Log.pre_parm = pre_parm;

%Save data
save(saveFileName,'data3', 'status3', 'trialLabel', 'runLabel', 'time_ms', 'Log');
fprintf('Save file %s : Done ... \n', saveFileName);
