%
% Section 4.4.1 figure4.17 Fisher's linear discriminant analysis (FLDA)
%
% 2011/03/01 OY
%
% Copyright (c) 2011, ATR Okito Yamashita, All rights reserved.

fprintf('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');
fprintf(' This program reproduces figure 4.17. Run many times.    \n');
fprintf('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');



clear 
close all

N = 50;
ro = 1;

S1 = [2 ro; ro 2];
S2 = [2 -1; -1 2];

mu1 = [2;1];
mu2 = [-2;-1];

%
% Data generation (from multivariate gaussian distribution)
%
z1 = randn(2,N);
z2 = randn(2,N);
   
STD1 = S1^(1/2);
STD2 = S2^(1/2);
    
x1 = STD1*z1+repmat(mu1,[1,N]);   % label 1
x2 = STD2*z2+repmat(mu2,[1,N]);   % label 2

figure,
plot(x1(1,:), x1(2,:), 'bo');
hold on
plot(x2(1,:), x2(2,:), 'rx');
hold on
axis equal

%
% FLDA
%
N1 = size(x1,2);
N2 = size(x1,2);

xbar1 = mean(x1,2);
xbar2 = mean(x2,2);
Sc = (N1*cov(x1') + N2*cov(x2'))/(N1+N2);  % within-class variation
Sb = (xbar1-xbar2)*(xbar1-xbar2)';         % between-class variation

% solve the generalized eigen problem
[V,D] = eig(Sb,Sc);

w = V(:,2);
w = w/norm(w);  % normalize w to be unit length
w = w*sign(w(1));  % 1st element is always positive

% another way to compute w
w1 = inv(Sc)*(xbar1-xbar2);
w1 = w1/norm(w1);
w1 = w1*sign(w1(1));  % 1st element is always positive


% add projection direction 
quiver(0,0, w(1)*5, w(2)*5, 'linewidth', 1, 'color', 'k');
quiver(0,0, -w(1)*5, -w(2)*5, 'linewidth', 1, 'color', 'k');


%
% project data x1 and x2 on w direction
%

y1 = w'*x1;
y2 = w'*x2;

figure,
plot(y1, zeros(length(y1),1), 'bo'); hold on; plot(y2, 0.1*ones(length(y2),1), 'rx'); 
axis([-inf inf -3 3]);
title('Data distribution on w direction');
%hist(y1); hold on; hist(y2); set(gca, 


