%
% Section 4.4.1 figure4.18 CSP filter : 2 dimension simulation data 
% See equation (4.7)
% 
% 2010.Oct modified by O. Yamashita
% 2010.Sep Takeda Yusuke
%
% Copyright (c) 2011, ATR Okito Yamashita, All rights reserved.

fprintf('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');
fprintf(' This program reproduces figure 4.18. Run many times.    \n');
fprintf('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');


close all
clear all

% Make original data
Ntrial=300;
x=[randn(Ntrial,1)*2,randn(Ntrial,1)];
y=[randn(Ntrial,1)*1,randn(Ntrial,1)*2];
x=x';% Class 1
y=y';% Class 2


% Make simulated EEG
w=randn(2,2);
wx=w*x;
wy=w*y;

%% Solving generalized eigen problem (4.7)

[V,D] = eig(cov(wx'), cov(wx')+cov(wy'));

d = diag(D);
d = d(2:-1:1);
W = [V(:,2) V(:,1)];  % descending order of D 
cspx = W'*wx;
cspy = W'*wy;

figure(1)
bar(d);
axis([-inf inf 0 1]);
title('Eigen Values')

figure(2)
subplot(1,2,1)
plot(wx(1,:),wx(2,:),'rx')
hold on;
axis equal
plot(wy(1,:),wy(2,:),'bo')
axis equal
title('Simulated EEG')
hold off
set(gca, 'xtick', [-4:2:4], 'ytick', [-4:2:4]);


subplot(1,2,2)
plot(cspx(1,:),cspx(2,:),'rx')
hold on;
plot(cspy(1,:),cspy(2,:),'bo')
title('After CSP')
axis equal
set(gca, 'xtick', [-2:1:2], 'ytick', [-2:1:2]);    

hold off


