%
% Section 4.4.1 supplement program :  Frequency resolution and the number of samples 
% (for the periodogram and multi-taper)
%
% 2010/10/11 OY
%
% Copyright (c) 2011, ATR Okito Yamashita, All rights reserved.

fprintf('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');
fprintf(' Frequency resolution of periodogram and multi-taper method \n');
fprintf('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');


clear 
close all

NN = [128 256 512];
Fs = 256;  % assumed sampling frequency

NW = 2;

%
% two sin wave
%

Fsin1 = 25.2;  % Hz
Fsin2 = 27.8;  % Hz

Tp1 = (1/Fsin1)/(1/Fs);  % period (time points)
Tp2 = (1/Fsin2)/(1/Fs);  % period (time points)

figure,
for ii = 1 : 3
    N = NN(ii);
    x = sin(2*pi/Tp1*[1:N]) + sin(2*pi/Tp2*[1:N]);

    [Pxp,F] = periodogram(x, [], [], Fs);
    [Pxm,F] = pmtm(x,NW,[],Fs);


    subplot(3,1,ii)
    plot(F, Pxp, 'k-');
    hold on;
    plot(F, Pxm, 'k--', 'linewidth', 2)
    hold on;
    plot([Fs/Tp1 Fs/Tp1], [0 max(Pxp)], 'k:');
    hold on
    plot([Fs/Tp2 Fs/Tp2], [0 max(Pxp)], 'k:');
    
    text(Fsin1-10, max(Pxp)*0.9, num2str(Fsin1), 'fontsize', 12);
    text(Fsin2+5, max(Pxp)*0.8, num2str(Fsin2), 'fontsize', 12);

    xlabel('Frequency (Hz)')
    axis([-inf inf 0 inf])


    if ii == 1, legend('Periodogram', 'MultiTaper'); end
    
    title(sprintf('Number of Data : %d , Frequency Resolution (multitaper) : %2.1f Hz, (periodogram) %2.1f Hz',  N, 2*NW/N*Fs, 2*Fs/N));
end


