%
% Section 4.5 figure4.24 Gauss Generative Classifier (GGC)
%
% 2011/02/25 OY
% 2010/10/11 OY
%
% Copyright (c) 2011, ATR Okito Yamashita, All rights reserved.

fprintf('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');
fprintf(' This program reproduces figure 4.24.         \n');
fprintf('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n');


clear
close all


RO= [-1  1];
strtitle = {'Data from S1=S2', 'Data from S1 ~= S2'};

for rr = 1 : length(RO),

    ro = RO(rr);

    S1 = [2 ro; ro 2];
    S2 = [2 -1; -1 2];

    mu1 = [4;2];
    mu2 = [-4;-2];

    N = 500;

    z1 = randn(2,N);
    z2 = randn(2,N);
   
    STD1 = S1^(1/2);
    STD2 = S2^(1/2);
    
    x1 = STD1*z1+repmat(mu1,[1,N]);
    x2 = STD2*z2+repmat(mu2,[1,N]);

    % learn gauss classifier
    
    [w0eq,w1eq,w2eq] = mygauss_classifier(x1,x2,1);  % S1 == S2
    [w0un,w1un,w2un] = mygauss_classifier(x1,x2,0);  % S1 ~= S2
    


    %%
    xmax = max([abs(x1(1,:)) abs(x2(1,:))]);
    ymax = max([abs(x1(2,:)) abs(x2(2,:))]);

    xmax = 10;
    ymax = 8;

    [X,Y] = meshgrid([-xmax:xmax/50:xmax], [-ymax:ymax/50:ymax]);

    XX = [X(:)'; Y(:)'];

    for ii = 1 : length(X(:));
      Ueq(ii) = XX(:,ii)'*w2eq*XX(:,ii) + w1eq*XX(:,ii)+w0eq ;
      Uun(ii) = XX(:,ii)'*w2un*XX(:,ii) + w1un*XX(:,ii)+w0un ;
    end

    figure,
    
    plot(x1(1,:), x1(2,:), 'ko');
    hold on
    plot(x2(1,:), x2(2,:), 'kx');
    hold on
    contour(X,Y,reshape(Ueq,[size(X,1),size(X,2)]),[0.000001], 'linewidth', 2);
    colormap('gray')
    hold on
    contour(X,Y,reshape(Uun,[size(X,1),size(X,2)]),[0.000001], 'linewidth', 3);  
    axis([-xmax xmax -ymax ymax])
    set(gca, 'ytick', [-8 -4 0 4 8])
    title(strtitle{rr});
    
    legend('Class1','Class2','GGC(equal cov)','GGC(unequal cov)');

end





    



