function Twindow = calc_timewindow(Tsample,Tperiod,Tnext,backmode)
%  Calculate indicies of beginning and end points of shifted time windows
%
% -- Example --
% Twindow = calc_timewindow(30, 20, 20) --> [1 20;21 30]
% Twindow = calc_timewindow(30, 20, 20, 1) --> [1 10;11 30]
%
% Copyright (c) 2011, ATR Okito Yamashita, All rights reserved.

if Tperiod < Tnext
    error('Caution : if Tperiod < Tnext, this code does not work correctly.');
end

if nargin < 4
    backmode = 0;
end

if backmode == 0   % Time window calculation from beginning
    Nperiod = ceil((Tsample - Tperiod)/Tnext + 1); % Number of periods
    Tbegin = 1:Tnext:((Nperiod-1)*Tnext + 1); % Start time index
    Tend	 = min(Tbegin + Tperiod - 1, Tsample);	% End time index

    Twindow  = [Tbegin' Tend'];

else    % Time window calculation from endpoint
    Nperiod = ceil((Tsample - Tperiod)/Tnext + 1); % Number of periods
    Tend	 = [Tsample:-Tnext:Tsample-(Nperiod-1)*Tnext];	% End time index
    Tbegin =   max(Tend - Tperiod+1, 1); 
   
    Twindow  = [sort(Tbegin') sort(Tend')];

end