function [powFeature] = calcfeat_power(pow, freq, FreqBand, powBase, freqBase, logScale)
%  
% Input 
%   pow : 4D matrix [Nch, Nfreq, Nt, Ntr]
%
% Output
%   powFeature : 4D matrix [Nch, Nfreqband, Nt, Ntr]
%
% 2010/05/18 OY
%
% Copyright (c) 2011, ATR Okito Yamashita, All rights reserved.

if nargin < 6,
    logScale = 0;
end

if nargin < 5 | freqBase == 0,
    freqBase = [];
end

if nargin < 4 | powBase == 0,
    powBase = [];
end
    

[Nch, Nfreq, Nt, Ntr] = size(pow);
Nfreqband = size(FreqBand, 1);

powFeature = zeros(Nch, Nfreqband, Nt, Ntr);
for fb = 1 : size(FreqBand,1)
    ixf = find(freq >= FreqBand(fb,1) & freq < FreqBand(fb,2));
    ixfBase = find(freqBase >= FreqBand(fb,1) & freqBase < FreqBand(fb,2));
    if isempty(ixfBase) | isempty(powBase)
        if logScale
            powFeature(:,fb,:,:) = log(mean(pow(:,ixf,:,:),2));
        else
            powFeature(:,fb,:,:) = mean(pow(:,ixf,:,:),2);
        end
    else     
        if logScale
             powFeature(:,fb,:,:) = log(mean(pow(:,ixf,:,:),2)) - log(mean(powBase(:,ixfBase,:,:),2));   
        else      
             powFeature(:,fb,:,:) = mean(pow(:,ixf,:,:),2) - mean(powBase(:,ixfBase,:,:),2);   
        end
    end
end
    
