function [cortr,corte,sc,cv_parm,IXTR,IXTE] = cls_nfoldcv(x,y,Nrun,method)
% N-fold cross validation
%
% Input
%   x : Ntrial * Nfeature  
% 
% 2011/02/25 OY
% * format of input feature vectors is modifed (transpose is required from
% older version).
% 
% 2010/08/02 OY
% 
% 2009/10/21 OY 
% * extended to multiclass problem case
% 2009/10/01 OY first version
%
% Copyright (c) 2011, ATR Okito Yamashita, All rights reserved.

if nargin < 4
    method = 'slr';
end


[Ntr,Nfeat] = size(x);
y = y(:); % column vector 

%
Ncv = Nrun;
Ntr_cv = floor(Ntr/Nrun);
if mod(Ntr,Nrun) ~= 0,
    warning('Each cross validation does not have the same number of test samples!');
end

% Cross-validation
sc = zeros(Nfeat,1);
cortr = zeros(Ncv,1);
corte = zeros(Ncv,1);

scale_mode = 'each';
mean_mode = 'each';

%
%
%
[IXTR, IXTE] = separate_nfoldcv(y, Nrun);

for cv = 1 : Ncv,

%    ixte = [1+Ntr_cv*(cv-1):Ntr_cv*cv];
%    ixtr = setdiff([1:Ntr], ixte);
    ixte = IXTE{cv};
    ixtr = IXTR{cv};

    xtr = x(ixtr,:);
    xte = x(ixte,:);
    ytr = y(ixtr);
    yte = y(ixte);

    if length(unique(y)) == 2,

        switch lower(method)
            case 'slr',

                if Nfeat < 1000
                   [ww, ix_eff, errT_tr, errT_te, cls_parm] = biclsfy_slrvar(xtr, ytr, xte, yte, 'nlearn', 1000, 'invhessian', 1);
                    sc(ix_eff{1}) = sc(ix_eff{1}) + 1;
                else
                    [ww, ix_eff, errT_tr, errT_te, cls_parm] = biclsfy_slrvar(xtr, ytr, xte, yte, 'nlearn', 1000, 'invhessian', 0);
                    sc(ix_eff{1}) = sc(ix_eff{1}) + 1;
                end

                % Obtain correct rate
                pertr = calc_percor(errT_tr);
                perte = calc_percor(errT_te);


            case 'classify',

                fprintf('--------------------------------------------------------\n');
                fprintf('   RUNNING linear Gaussian Classifier \n');
                fprintf('--------------------------------------------------------\n');

                [xtr, scale, base] = normalize_feature(xtr, scale_mode, mean_mode);
                [xte, scale, base] = normalize_feature(xte, scale_mode, mean_mode, scale, base);

                if license('test', 'statistics_toolbox')
                    [yteh, err] = classify(xte, xtr, ytr, 'linear');
                else
                    [w0,w1,w2,err,yteh] = mygauss_classifier(xtr(ytr==1,:)',xtr(ytr==2,:)',1,xte');
                end
                
                % Obtain correct rate
                pertr = (1 - err)*100;
                perte = length(find(yte == yteh))/length(yte)*100;

                cls_parm = struct;
                
            case 'svm',
                fprintf('--------------------------------------------------------\n');
                fprintf('   RUNNING Linear SVM (Binary version) \n')
                fprintf('--------------------------------------------------------\n');

                [xtr, scale, base] = normalize_feature(xtr, scale_mode, mean_mode);
                [xte, scale, base] = normalize_feature(xte, scale_mode, mean_mode, scale, base);
                
                ops = sprintf('-t 0 -c 1 -r 0 -d 3 -b 1');
                ops2 = sprintf('-b 1');

                % Train:
                Model = svmtrain(ytr, xtr, ops);
                [preds, accur, decVals] = svmpredict(ytr, xtr, Model, ops2);
                pertr = accur(1);
                % Test 
                [preds, accur, decVals] = svmpredict(yte, xte, Model, ops2);
                perte = accur(1);
                
                cls_parm = struct;
                
        end

    else
        [ww, ix_eff, errT_tr, errT_te, cls_parm] = muclsfy_slrvarovo(xtr, ytr, xte, yte, 'nlearn', 1000, 'invhessian', 1);

        for cls = 1 : length(unique(y))
            sc(ix_eff{cls}) = sc(ix_eff{cls}) + 1;
        end
    end
    

    % Obtain correct rate
    cortr(cv)= pertr;
    corte(cv)= perte;
    

end

cv_parm.cls_parm = cls_parm;
cv_parm.Nrun = Nrun;
