function multiplot5(haxes, x, y, range, col, lwidth, setaxis, nch4hline, ylabel)
% PLOT MULTIPLE TIME SERIES IN ONE AXIS
%
% Syntax
% > multiplot5(haxes, x, y, range, color, lwidth, setaxis, nch4hline, ylabel)
% 
% Example
% > multiplot5(gca, x, y, [-20 20], [], 2) 
%
% Input 
%   x : Nt*1 or 1*Nt
%   y : Nch * Nt
%  range : yrange of each channel 
%  col : structure containing channel-color correspondence table 
%    .ixchan 
%    .color
%   example. col.ixchan = {[1:10],[11:20]}; col.color = {'b', 'r'};    
%
%
% Required function : xyrefline.m
%
% 2010/08/23 OY ver.5
% * add 'ylabel' in input arguments. yticklabel can be set by this. 
% 2010/08/02 OY ver.4
% * bug fix in lines where 'col' struct is initialized (empty 'col' case)
% 2010/03/17 OY ver.4
% * axes handle is required
% 2010/03/16 OY ver.3
% * axis range is modified
% * specific color for some specific channels  
% 2009/12/21 OY ver.2
% * 'x' is required 
% * horizontal line can be added
%
% Copyright (c) 2011, ATR Okito Yamashita, All rights reserved.

if nargin < 9
    ylabel = [];
end

if nargin < 8
    nch4hline = [];
end

if nargin < 7,
    setaxis = 1;
end

if nargin < 6,
    lwidth = 1;
end

if nargin < 5 | isempty(col),
    for ch = 1 :size(y,1)
    col.ixchan{ch} = num2str(ch);
    col.color{ch} = 'k';
    end
end

if nargin < 4 | isempty(range)
    range(2) = median(y(:));
    range(1) = min(y(:));
end


[Nch,Nt] = size(y);
z = zeros(Nch, Nt);

miny = range(1);
maxy = range(2);

basey = (maxy - miny) * 1.2;

for ch = 1 : Nch 
    z(ch,:) = y(ch,:) +  basey * (Nch-ch);
   % label{ch} = num2str(Nch-ch+1);
end

axes(haxes);
% Plot data 
for ch = 1 : Nch
    for jj = 1 : length(col), 
        if ismember(ch, col.ixchan{jj}), color = col.color{jj}; 
        else color = 'K'; end
    end
    plot(x, z(ch,:)', color, 'linewidth', lwidth);
    hold on
end
hold off

if setaxis, axis([-inf inf -basey basey*Nch]); end

if ~iscell(ylabel) & isemtpy(ylabel),
    set(gca, 'yticklabel', []);
else
    set(gca, 'ytick', [0:basey:basey*(Nch-1)], 'yticklabel', ylabel);
end

if ~isempty(nch4hline),
    yvals = (Nch - nch4hline) * basey;
    xyrefline([], yvals, 'Linestyle', ':');
end


    
    
    
