function [w0,w1,w2,perr,ytest] = mygauss_classifier(x1,x2,mode,xtest)
% THIS IS LIMITED VERSION OF classify.m 
% * binary classification
% * equal prior probability  
% 
% Input
%   x1 : feature matrix of class1  D*N1
%   x2 : feature matrix of class1  D*N2
%   xtest : feature matrix for test
%   mode : 1 = equal covariance  otherwise = unequal covariance 
% Output
%   w0,w1,w2 : classifier coefficients 
%
% 2011/03/17 OY
% * add routine for calculating correct classification performance
% 2011/02/25 OY
%
% Copyright (c) 2011, ATR Okito Yamashita, All rights reserved.

if nargin < 4
    dotest = 0;
end


if mode == 1
    fprintf('Equal Covariance ...\n');
    
    
    % MLE of mu1, mu2, S1 and S2
    N1 = size(x1,2);
    N2 = size(x2,2);
    
    mu1e = mean(x1,2);
    mu2e = mean(x2,2);
    S1e = cov(x1');
    S2e = cov(x2');
    Se = (N1*S1e+N2*S2e)/(N1+N2);
    
    % Bayes boundary
    w2 = 0;
    w1 = -2*(mu1e - mu2e)'*inv(Se);
    w0 = mu1e'*inv(Se)*mu1e - mu2e'*inv(Se)*mu2e;

else
    fprintf('Unequal Covariance ...\n');
    

    % MLE of mu1, mu2, S1 and S2
    N1 = size(x1,2);
    N2 = size(x2,2);
    
    mu1e = mean(x1,2);
    mu2e = mean(x2,2);
    S1e = cov(x1');
    S2e = cov(x2');
    
    % Bayes boundary
    w2 = inv(S1e) - inv(S2e);
    w1 = -2*(mu1e'*inv(S1e) - mu2e'*inv(S2e));
    w0 = mu1e'*inv(S1e)*mu1e - mu2e'*inv(S2e)*mu2e + log(det(S1e)) - log(det(S2e));

end


%
% Performance 
%
nerr = 0;
for ii = 1 : N1
    f = w0 + w1*x1(:,ii)+x1(:,ii)'*w2*x1(:,ii);
    if f > 0
      nerr = nerr + 1;
    end
end
for ii = 1 : N2
    f = w0 + w1*x2(:,ii)+x1(:,ii)'*w2*x2(:,ii);
    if f < 0
      nerr = nerr + 1;
    end
end
perr = nerr/(N1+N2); 


%
% Test data
%
if dotest
    ytest = zeros(size(xtest,2),1);
    for ii = 1 : size(xtest,2)
        f = w0 + w1*xtest(:,ii)+xtest(:,ii)'*w2*xtest(:,ii);
        if f < 0
            ytest(ii) = 1;
        else
            ytest(ii) = 2;
        end
    end
else
    ytest = [];
end
