function [data, B, A] = pre_highpass(data, samplefreq, highpassfreq, filtertype, order)
% Highpass filtering
% 
% -- Output
% data : bandpassed filtered data
%  B   : filter weights (numerator parts)
%  A   : filter weights (denominator parts). If IIR filter is used, A is empty.
%
% 2011/02/25 OY
% * remove several lines for this book 
% 2009/11/20 OY
% * nothing done when filtertype = [];
% 2009/09/02 OY
%
% Copyright (c) 2011, ATR Okito Yamashita, All rights reserved.

[Nch,Nt] = size(data);

if isempty(filtertype),
    data = data;
    B = [];
    A = [];
    return
end

switch filtertype
    case 'butter'
        [B,A] = butter(order, highpassfreq/(samplefreq/2),'high');
        for nch = 1 : Nch,
            data(nch,:) = filtfilt(B,A,data(nch,:));
        end
    case 'onlinebutter',
        %         [A,B,C,D] = butter(order, highpassfreq/(samplefreq/2),'high');
        %         A = A'; B = B'; C = C'; D = D';
        %         Y = zeros( Nch, Nt);
        %         X = zeros( Nch, order );
        %         for nt=1:Nt
        %             Y(:,nt) = X * C + data(:,nt) * D;
        %             X      = X * A + data(:,nt) * B;
        %         end
        %         data=Y;
        [B,A] = butter(order, highpassfreq/(samplefreq/2),'high');
        for nch = 1 : Nch,
            data(nch,:) = filter(B,A,data(nch,:));
        end

        
    otherwise
        error('No filtertype. [''butter'', ''onlinebutter'']');
end